@@ -22,16 +22,17 @@ class gNMIclient(object):
     This class instantiates the object, which interacts with the network elements over gNMI.
     """
     def __init__(self, target: tuple, username: str = None, password: str = None, 
-                 to_print: bool = False, insecure: bool = False, path_cert: str = None):
+                 debug: bool = False, insecure: bool = False, path_cert: str = None, override: str = None):
         """
         Initializing the object
         """
         self.__metadata = [('username', username), ('password', password)]
         self.__capabilities = None
-        self.__to_print = to_print
+        self.__debug = debug
         self.__insecure = insecure
         self.__path_cert = path_cert
-        self.__options=[('grpc.ssl_target_name_override', target[0])]
+        self.__override = override
+        self.__options=[('grpc.ssl_target_name_override', self.__override)]
 
         if re.match('.*:.*', target[0]):
             self.__target = (f'[{target[0]}]', target[1])
@@ -77,8 +78,12 @@ class gNMIclient(object):
             gnmi_message_request = CapabilityRequest()
             gnmi_message_response = self.__stub.Capabilities(gnmi_message_request, metadata=self.__metadata)
 
-            if self.__to_print:
+            if self.__debug:
+                print("gNMI request:\n------------------------------------------------")
+                print(gnmi_message_request)
+                print("------------------------------------------------\n\n\ngNMI response:\n------------------------------------------------")
                 print(gnmi_message_response)
+                print("------------------------------------------------")
 
             if gnmi_message_response:
                 response = {}
@@ -166,7 +171,7 @@ class gNMIclient(object):
         if encoding in encoding_set:
             if encoding.lower() == 'json':
                 pb_encoding = 0
-            if encoding.lower() == 'bytes':
+            elif encoding.lower() == 'bytes':
                 pb_encoding = 1
             elif encoding.lower() == 'proto':
                 pb_encoding = 2
@@ -176,7 +181,11 @@ class gNMIclient(object):
                 pb_encoding = 4
 
         try:
-            protobuf_paths = [gnmi_path_generator(pe) for pe in path]
+            if not path:
+                protobuf_paths = []
+                protobuf_paths.append(gnmi_path_generator(path))
+            else:
+                protobuf_paths = [gnmi_path_generator(pe) for pe in path]
 
         except:
             logging.error(f'Conversion of gNMI paths to the Protobuf format failed')
@@ -192,8 +201,12 @@ class gNMIclient(object):
             gnmi_message_request = GetRequest(path=protobuf_paths, type=pb_datatype, encoding=pb_encoding)
             gnmi_message_response = self.__stub.Get(gnmi_message_request, metadata=self.__metadata)
 
-            if self.__to_print:
+            if self.__debug:
+                print("gNMI request:\n------------------------------------------------")
+                print(gnmi_message_request)
+                print("------------------------------------------------\n\n\ngNMI response:\n------------------------------------------------")
                 print(gnmi_message_response)
+                print("------------------------------------------------")
 
             if gnmi_message_response:
                 response = {}
@@ -263,7 +276,7 @@ class gNMIclient(object):
             return None
 
 
-    def set(self, delete: list = None, replace: list = None, update: list = None):
+    def set(self, delete: list = None, replace: list = None, update: list = None, encoding: str = 'json'):
         """
         Changing the configuration on the destination network elements.
         Could provide a single attribute or multiple attributes.
@@ -278,10 +291,22 @@ class gNMIclient(object):
         replace:
           - list of tuples where the first entry path provided as a string, and the second entry
             is a dictionary with the configuration to be configured
+
+        The encoding argument may have the following values per gNMI specification:
+          - json
+          - bytes
+          - proto
+          - ascii
+          - json_ietf
         """
         del_protobuf_paths = []
         replace_msg = []
         update_msg = []
+        encoding_set = {'json', 'bytes', 'proto', 'ascii', 'json_ietf'}
+
+        if encoding not in encoding_set:
+            logging.error(f'The encoding {encoding} is not supported. The allowed are: {", ".join(encoding_set)}.')
+            sys.exit(11)
 
         if delete:
             if isinstance(delete, list):
@@ -303,7 +328,16 @@ class gNMIclient(object):
                         u_path = gnmi_path_generator(ue[0])
                         u_val = json.dumps(ue[1]).encode('utf-8')
 
-                        replace_msg.append(Update(path=u_path, val=TypedValue(json_val=u_val)))
+                        if encoding == 'json':
+                            replace_msg.append(Update(path=u_path, val=TypedValue(json_val=u_val)))
+                        elif encoding == 'bytes':
+                            replace_msg.append(Update(path=u_path, val=TypedValue(bytes_val=u_val)))
+                        elif encoding == 'proto':
+                            replace_msg.append(Update(path=u_path, val=TypedValue(proto_bytes=u_val)))
+                        elif encoding == 'ascii':
+                            replace_msg.append(Update(path=u_path, val=TypedValue(ascii_val=u_val)))
+                        elif encoding == 'json_ietf':
+                            replace_msg.append(Update(path=u_path, val=TypedValue(json_ietf_val=u_val)))
 
                     else:
                         logging.error(f'The input element for Update message must be tuple, got {ue}.')
@@ -320,7 +354,16 @@ class gNMIclient(object):
                         u_path = gnmi_path_generator(ue[0])
                         u_val = json.dumps(ue[1]).encode('utf-8')
 
-                        update_msg.append(Update(path=u_path, val=TypedValue(json_val=u_val)))
+                        if encoding == 'json':
+                            update_msg.append(Update(path=u_path, val=TypedValue(json_val=u_val)))
+                        elif encoding == 'bytes':
+                            update_msg.append(Update(path=u_path, val=TypedValue(bytes_val=u_val)))
+                        elif encoding == 'proto':
+                            update_msg.append(Update(path=u_path, val=TypedValue(proto_bytes=u_val)))
+                        elif encoding == 'ascii':
+                            update_msg.append(Update(path=u_path, val=TypedValue(ascii_val=u_val)))
+                        elif encoding == 'json_ietf':
+                            update_msg.append(Update(path=u_path, val=TypedValue(json_ietf_val=u_val)))
 
                     else:
                         logging.error(f'The input element for Update message must be tuple, got {ue}.')
@@ -331,14 +374,15 @@ class gNMIclient(object):
                 sys.exit(10)
 
         try:
-            if self.__to_print:
-                print(SetRequest(delete=del_protobuf_paths, update=update_msg, replace=replace_msg))
-
             gnmi_message_request = SetRequest(delete=del_protobuf_paths, update=update_msg, replace=replace_msg)
             gnmi_message_response = self.__stub.Set(gnmi_message_request, metadata=self.__metadata)
 
-            if self.__to_print:
+            if self.__debug:
+                print("gNMI request:\n------------------------------------------------")
+                print(gnmi_message_request)
+                print("------------------------------------------------\n\n\ngNMI response:\n------------------------------------------------")
                 print(gnmi_message_response)
+                print("------------------------------------------------")
 
             if gnmi_message_response:
                 response = {}
@@ -567,6 +611,11 @@ class gNMIclient(object):
                 logging.error('Subscribe subscribe requst is specified, but the value is not list.')
 
 
+            if self.__debug:
+                print("gNMI request:\n------------------------------------------------")
+                print(gnmi_message_request)
+                print("------------------------------------------------")
+
         return self.__stub.Subscribe(self.__generator(gnmi_message_request), metadata=self.__metadata)
 
 
@@ -592,6 +641,11 @@ def telemetryParser(in_message=None):
     The telemetry parser is method used to covert the Protobuf message
     """
 
+    if self.__debug:
+        print("gNMI response:\n------------------------------------------------")
+        print(in_message)
+        print("------------------------------------------------")
+
     try:
         response = {}
         if in_message.HasField('update'):
